@@ -29,12 +29,21 @@
  */
 
 
-#define MODULE_VERSION  "1.13"
+#define MODULE_VERSION  "1.15"
 
 #define PY_SSIZE_T_CLEAN
 
 #include <Python.h>
+#include <string.h>
+#include <lzo/lzo1.h>
+#include <lzo/lzo1a.h>
+#include <lzo/lzo1b.h>
+#include <lzo/lzo1c.h>
+#include <lzo/lzo1f.h>
 #include <lzo/lzo1x.h>
+#include <lzo/lzo1y.h>
+#include <lzo/lzo1z.h>
+#include <lzo/lzo2a.h>
 
 /* Python 2x3 compatible macros */
 #if PY_VERSION_HEX >= 0x03000000
@@ -61,21 +70,26 @@
 
 static PyObject *LzoError;
 
+// custom function type definitions to allow compatibility of various algorithms
+typedef int (*lzo_compress_fn)(const lzo_bytep, lzo_uint, lzo_bytep, lzo_uintp, lzo_voidp);
+typedef int (*lzo_decompress_fn)(const lzo_bytep, lzo_uint, lzo_bytep, lzo_uintp, lzo_voidp /* NOT USED */);
 
 /***********************************************************************
 // compress
 ************************************************************************/
 
 static /* const */ char compress__doc__[] =
-"compress(string[,level[,header]]) -- Compress string, returning a string "
+"compress(string[,level[,header[,algorithm]]]) -- Compress string, returning a string "
 "containing compressed data.\n"
 "level  - Set compression level of either 1 (default) or 9.\n"
 "header - Include metadata header for decompression in the output "
 "(default: True).\n"
+"algorithm (keyword argument)  - can be either LZO1, LZO1A, LZO1B, LZO1C, LZO1F, LZO1X, LZO1Y, LZO1Z, LZO2A."
+"(default: LZO1X).\n"
 ;
 
 static PyObject *
-compress(PyObject *dummy, PyObject *args)
+compress(PyObject *dummy, PyObject *args, PyObject *kwds)
 {
     PyObject *result_str;
     lzo_voidp wrkmem = NULL;
@@ -90,9 +104,16 @@ compress(PyObject *dummy, PyObject *args)
     int header = 1;
     int err;
 
+    static char* argnames[] = {"", "", "", "algorithm", NULL};
+    char *algorithm = "LZO1X";
+    lzo_compress_fn compress_1_ptr;
+    lzo_compress_fn compress_999_ptr;
+    lzo_uint32_t MEM_COMPRESS_1;
+    lzo_uint32_t MEM_COMPRESS_999;
+
     /* init */
     UNUSED(dummy);
-    if (!PyArg_ParseTuple(args, "s#|ii", &in, &len, &level, &header))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s#|ii$s", argnames, &in, &len, &level, &header, &algorithm))
         return NULL;
     if (len < 0)
         return NULL;
@@ -107,6 +128,80 @@ compress(PyObject *dummy, PyObject *args)
       return NULL;
     }
 
+    /* init for different algorithms*/
+    if (strcmp(algorithm, "LZO1") == 0) {
+      // settings specific for lzo1
+      MEM_COMPRESS_1 = LZO1_MEM_COMPRESS;
+      MEM_COMPRESS_999 = LZO1_99_MEM_COMPRESS;
+
+      compress_1_ptr = &lzo1_compress;
+      compress_999_ptr = &lzo1_99_compress;
+    }
+    else if (strcmp(algorithm, "LZO1A") == 0){
+      // settings for LZO1A
+      MEM_COMPRESS_1 = LZO1A_MEM_COMPRESS;
+      MEM_COMPRESS_999 = LZO1A_99_MEM_COMPRESS;
+
+      compress_1_ptr = &lzo1a_compress;
+      compress_999_ptr = &lzo1a_99_compress;
+    }
+    else if (strcmp(algorithm, "LZO1B") == 0){
+      // settings for LZO1B
+      MEM_COMPRESS_1 = LZO1B_MEM_COMPRESS;
+      MEM_COMPRESS_999 = LZO1B_999_MEM_COMPRESS;
+
+      compress_1_ptr = &lzo1b_1_compress;
+      compress_999_ptr = &lzo1b_999_compress;
+    }
+    else if (strcmp(algorithm, "LZO1C") == 0){
+      // settings for LZO1C
+      MEM_COMPRESS_1 = LZO1C_MEM_COMPRESS;
+      MEM_COMPRESS_999 = LZO1C_999_MEM_COMPRESS;
+
+      compress_1_ptr = &lzo1c_1_compress;
+      compress_999_ptr = &lzo1c_999_compress;
+    }
+    else if (strcmp(algorithm, "LZO1F") == 0){
+      // settings for LZO1F
+      MEM_COMPRESS_1 = LZO1F_MEM_COMPRESS;
+      MEM_COMPRESS_999 = LZO1F_999_MEM_COMPRESS;
+
+      compress_1_ptr = &lzo1f_1_compress;
+      compress_999_ptr = &lzo1f_999_compress;
+    }
+    else if (strcmp(algorithm, "LZO1Y") == 0){
+      // settings for LZO1Y
+      MEM_COMPRESS_1 = LZO1Y_MEM_COMPRESS;
+      MEM_COMPRESS_999 = LZO1Y_999_MEM_COMPRESS;
+
+      compress_1_ptr = &lzo1y_1_compress;
+      compress_999_ptr = &lzo1y_999_compress;
+    }
+    else if (strcmp(algorithm, "LZO1Z") == 0){
+      // settings for LZO1Z
+      MEM_COMPRESS_1 = LZO1Z_999_MEM_COMPRESS;
+      MEM_COMPRESS_999 = LZO1Z_999_MEM_COMPRESS;
+
+      compress_1_ptr = &lzo1z_999_compress;
+      compress_999_ptr = &lzo1z_999_compress;
+    }
+    else if (strcmp(algorithm, "LZO2A") == 0){
+      // settings for LZO2A
+      MEM_COMPRESS_1 = LZO2A_999_MEM_COMPRESS;
+      MEM_COMPRESS_999 = LZO2A_999_MEM_COMPRESS;
+
+      compress_1_ptr = &lzo2a_999_compress;
+      compress_999_ptr = &lzo2a_999_compress;
+    }
+    else {
+      // settings for LZO1X
+      MEM_COMPRESS_1 = LZO1X_1_MEM_COMPRESS;
+      MEM_COMPRESS_999 = LZO1X_999_MEM_COMPRESS;
+
+      compress_1_ptr = &lzo1x_1_compress;
+      compress_999_ptr = &lzo1x_999_compress;
+    }
+
     in_len = len;
     out_len = in_len + in_len / 16 + 64 + 3;
 
@@ -119,9 +214,9 @@ compress(PyObject *dummy, PyObject *args)
     if (result_str == NULL)
         return PyErr_NoMemory();
     if (level == 1)
-        wrkmem = (lzo_voidp) PyMem_Malloc(LZO1X_1_MEM_COMPRESS);
+        wrkmem = (lzo_voidp) PyMem_Malloc(MEM_COMPRESS_1);
     else
-        wrkmem = (lzo_voidp) PyMem_Malloc(LZO1X_999_MEM_COMPRESS);
+        wrkmem = (lzo_voidp) PyMem_Malloc(MEM_COMPRESS_999);
     if (wrkmem == NULL)
     {
         Py_DECREF(result_str);
@@ -142,13 +237,13 @@ compress(PyObject *dummy, PyObject *args)
     {
         if (header)
             out[0] = 0xf0;
-        err = lzo1x_1_compress(in, in_len, outc, &new_len, wrkmem);
+        err = (*compress_1_ptr)(in, in_len, outc, &new_len, wrkmem);
     }
     else
     {
         if (header)
             out[0] = 0xf1;
-        err = lzo1x_999_compress(in, in_len, outc, &new_len, wrkmem);
+        err = (*compress_999_ptr)(in, in_len, outc, &new_len, wrkmem);
     }
     Py_END_ALLOW_THREADS
 
@@ -186,14 +281,16 @@ compress(PyObject *dummy, PyObject *args)
 ************************************************************************/
 
 static /* const */ char decompress__doc__[] =
-"decompress(string[,header[,buflen]]) -- Decompress the data in string, returning a string containing the decompressed data.\n"
+"decompress(string[,header[,buflen[,algorithm]]]) -- Decompress the data in string, returning a string containing the decompressed data.\n"
 "header - Metadata header is included in input (default: True).\n"
 "buflen - If header is False, a buffer length in bytes must be given that "
 "will fit the output.\n"
+"algorithm (keyword argument) - can be either LZO1, LZO1A, LZO1B, LZO1C, LZO1F, LZO1X, LZO1Y, LZO1Z, LZO2A."
+"(default: LZO1X).\n"
 ;
 
 static PyObject *
-decompress(PyObject *dummy, PyObject *args)
+decompress(PyObject *dummy, PyObject *args, PyObject *kwds)
 {
     PyObject *result_str;
     const lzo_bytep in;
@@ -206,17 +303,21 @@ decompress(PyObject *dummy, PyObject *args)
     int header = 1;
     int err;
 
+    static char* argnames[] = {"", "", "", "algorithm", NULL};
+    char *algorithm = "LZO1X";
+    lzo_decompress_fn decompress_ptr;
+
     /* init */
     UNUSED(dummy);
-    if (!PyArg_ParseTuple(args, "s#|ii", &in, &len, &header, &buflen))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s#|ii$s", argnames, &in, &len, &header, &buflen, &algorithm))
         return NULL;
     if (header) {
         if (len < 5 + 3 || in[0] < 0xf0 || in[0] > 0xf1)
             goto header_error;
-        out_len = (in[1] << 24) | (in[2] << 16) | (in[3] << 8) | in[4];
+        out_len = ((lzo_uint)in[1] << 24) | (in[2] << 16) | (in[3] << 8) | in[4];
         in_len = len - 5;
         in += 5;
-        if ((int)out_len < 0 || in_len > out_len + out_len / 64 + 16 + 3)
+        if (out_len < 0 || in_len > out_len + out_len / 64 + 16 + 3)
             goto header_error;
     }
     else {
@@ -225,6 +326,44 @@ decompress(PyObject *dummy, PyObject *args)
         in_len = len;
     }
 
+    /* init for different algorithms*/
+    if (strcmp(algorithm, "LZO1") == 0) {
+      // settings specific for lzo1
+      decompress_ptr = &lzo1_decompress;
+    }
+    else if (strcmp(algorithm, "LZO1A") == 0){
+      // settings for LZO1A
+      decompress_ptr = &lzo1a_decompress;
+    }
+    else if (strcmp(algorithm, "LZO1B") == 0){
+      // settings for LZO1B
+      decompress_ptr = &lzo1b_decompress_safe;
+    }
+    else if (strcmp(algorithm, "LZO1C") == 0){
+      // settings for LZO1C
+      decompress_ptr = &lzo1c_decompress_safe;
+    }
+    else if (strcmp(algorithm, "LZO1F") == 0){
+      // settings for LZO1F
+      decompress_ptr = &lzo1f_decompress_safe;
+    }
+    else if (strcmp(algorithm, "LZO1Y") == 0){
+      // settings for LZO1Y
+      decompress_ptr = &lzo1y_decompress_safe;
+    }
+    else if (strcmp(algorithm, "LZO1Z") == 0){
+      // settings for LZO1Z
+      decompress_ptr = &lzo1z_decompress_safe;
+    }
+    else if (strcmp(algorithm, "LZO2A") == 0){
+      // settings for LZO2A
+      decompress_ptr = &lzo2a_decompress_safe;
+    }
+    else {
+      // settings for LZO1X
+      decompress_ptr = &lzo1x_decompress_safe;
+    }
+
     /* alloc buffers */
 #if PY_MAJOR_VERSION >= 3
     result_str = PyBytes_FromStringAndSize(NULL, out_len);
@@ -243,7 +382,7 @@ decompress(PyObject *dummy, PyObject *args)
 
     Py_BEGIN_ALLOW_THREADS
     new_len = out_len;
-    err = lzo1x_decompress_safe(in, in_len, out, &new_len, NULL);
+    err = (*decompress_ptr)(in, in_len, out, &new_len, NULL);
     Py_END_ALLOW_THREADS
 
     if (err != LZO_E_OK || (header && new_len != out_len) )
@@ -427,9 +566,9 @@ crc32(PyObject *dummy, PyObject *args)
 static /* const */ PyMethodDef methods[] =
 {
     {"adler32",    (PyCFunction)adler32,    METH_VARARGS, adler32__doc__},
-    {"compress",   (PyCFunction)compress,   METH_VARARGS, compress__doc__},
+    {"compress",   (PyCFunction)compress,   METH_VARARGS | METH_KEYWORDS, compress__doc__},
     {"crc32",      (PyCFunction)crc32,      METH_VARARGS, crc32__doc__},
-    {"decompress", (PyCFunction)decompress, METH_VARARGS, decompress__doc__},
+    {"decompress", (PyCFunction)decompress, METH_VARARGS | METH_KEYWORDS, decompress__doc__},
     {"optimize",   (PyCFunction)optimize,   METH_VARARGS, optimize__doc__},
     {NULL, NULL, 0, NULL}
 };
